#version 430 core
/*
* Copyright 2019 Vienna University of Technology.
* Institute of Computer Graphics and Algorithms.
* This file is part of the ECG Lab Framework and must not be redistributed.
*/
layout (location = 0) out vec4 color;
layout (location = 1) out vec4 brightThreshold;

in VertexData {
	vec3 position_world;
	vec3 normal_world;
	vec2 uv;
	vec4 fragPosLight;
} vert;

uniform vec3 camera_world;

uniform vec3 materialCoefficients; // x = ambient, y = diffuse, z = specular 
uniform float specularAlpha;
uniform sampler2D diffuseTexture;
uniform sampler2D shadowMap;

uniform struct DirectionalLight {
	vec3 color;
	vec3 direction;
} dirL;

uniform struct PointLight {
    vec3 color;
    vec3 position;
    vec3 attenuation;
};

#define nr_of_point_lights 10
uniform PointLight pointL[nr_of_point_lights];


vec3 phong(vec3 n, vec3 l, vec3 v, vec3 diffuseC, float diffuseF, vec3 specularC, float specularF, float alpha, bool attenuate, vec3 attenuation) {
	float d = length(l);
	l = normalize(l);
	float att = 1.0;	
	if(attenuate) att = 1.0f / (attenuation.x + d * attenuation.y + d * d * attenuation.z);
	vec3 r = reflect(-l, n);
	return (diffuseF * diffuseC * max(0, dot(n, l)) + specularF * specularC * pow(max(0, dot(r, v)), alpha)) * att; 
}

void main() {	
	vec3 n = normalize(vert.normal_world);
	vec3 v = normalize(camera_world - vert.position_world);
	
	vec3 texColor = texture(diffuseTexture, vert.uv).rgb;
	color = vec4(texColor * materialCoefficients.x, 0.5); // ambient
	
	//
//	float shadow = 0.0f;
//	vec3 lCoords = vert.fragPosLight.xyz / vert.fragPosLight.w;
//	if(lCoords.z <= 1.0f){
//		lCoords = (lCoords + 1.0f) /2.0f;
//		float closestDepth = texture(shadowMap, lCoords.xy).r;
//		float currentDepth = lCoords.z;
//
//		float bias = 0.005f;
//		if(currentDepth > closestDepth + bias)
//			shadow = 1.0f;
//	}

	float shadow = 0.0f;
	// Sets lightCoords to cull space
	vec3 lightCoords = vert.fragPosLight.xyz / vert.fragPosLight.w;
	if(lightCoords.z <= 1.0f)
	{
		// Get from [-1, 1] range to [0, 1] range just like the shadow map
		lightCoords = (lightCoords + 1.0f) / 2.0f;
		float currentDepth = lightCoords.z;
		// Prevents shadow acne
		float bias = max(0.0005 * (1.0 - dot(normalize(vert.normal_world), dirL.direction)), 0.005);
		//float bias = 0.005f;

		// Smoothens out the shadows
		int sampleRadius = 2;
		vec2 pixelSize = 1.0 / textureSize(shadowMap, 0);
		for(int y = -sampleRadius; y <= sampleRadius; y++)
		{
		    for(int x = -sampleRadius; x <= sampleRadius; x++)
		    {
		        float closestDepth = texture(shadowMap, lightCoords.xy + vec2(x, y) * pixelSize).r;
				if (currentDepth > closestDepth + bias)
					shadow += 1.0f;     
		    }    
		}
		// Get average shadow
		shadow /= pow((sampleRadius * 2 + 1), 2);
	}


	// add directional light contribution
	color.rgb += phong(n, dirL.direction, v, dirL.color * texColor, materialCoefficients.y * (1.0f - shadow), dirL.color, materialCoefficients.z * (1.0f - shadow), specularAlpha, false, vec3(0));


			
	// add point light contribution
    for(int i = 0; i < nr_of_point_lights; i++){
        color.rgb += phong(n, pointL[i].position - vert.position_world, v, pointL[i].color * texColor, materialCoefficients.y, pointL[i].color, materialCoefficients.z, specularAlpha, true, pointL[i].attenuation);
    }

	float brightness = dot(color.rgb, vec3(0.2126, 0.7152, 0.0722));
    if(brightness > 1.0)
        brightThreshold = color;
    else
        brightThreshold = vec4(0.0, 0.0, 0.0, 1.0);
}

